/***************************************************************************
 *   Copyright (C) 2007 by Marcin Swiechowicz   *
 *   marcin.swiechowicz@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "kdcapplysubtitlejob.h"

#include "../kdcconfig.h"
#include "../kdcfilefactory.h"
#include "../kdcdisc.h"

#include <QStringList>
#include <QByteArray>
#include <QFile>

#include <KLocale>

KDCApplySubtitleJob::KDCApplySubtitleJob( int id, const KDCMovie *movie, QObject *parent )
	: KDCJob( parent )
{
	p_id = id + 1;
	p_fileName = KDCFileFactory::movieName( movie );
	p_oldFrame = 0;
	setTotalSize( qRound( movie->videoStream()->metaData()->length()/1000.0*movie->fps() ) );
	p_movie = movie;

	p_process = new KProcess( this );
	p_process->setOutputChannelMode( KProcess::SeparateChannels );
	p_process->setWorkingDirectory( KDCFileFactory::discDirectory( movie->disc() ) );
	qDebug( KDCFileFactory::discDirectory( movie->disc() ).toUtf8() );
	p_process->setStandardInputFile( p_fileName );
	qDebug( p_fileName.toUtf8() );
	p_fileName.chop( 4 );
	p_process->setStandardOutputFile( p_fileName + "_sub.mpg" );

	connect( p_process, SIGNAL( readyReadStandardError() ), this, SLOT( on_readyReadStandardError() ) );
	connect( p_process, SIGNAL( finished( int , QProcess::ExitStatus ) ), this, SLOT( on_finished( int , QProcess::ExitStatus ) ) );

	QStringList arguments;
	arguments << "-n" << QString::number( KDCConfig::niceLevel() );
	arguments << "spumux" << "-m";
	if ( movie->disc()->discType() == KDC::DVD )
		arguments << "dvd";
	else
		if ( movie->disc()->discType() == KDC::CVD )
			arguments << "cvd";
	else
		arguments << "svcd";

	arguments << "-s" << QString::number( id ) << "-P";
	arguments << KDCFileFactory::movieSubtitleConfigFile( movie, p_id );
	qDebug( KDCFileFactory::movieSubtitleConfigFile( movie, p_id ).toUtf8() );
	p_process->setProgram( "nice", arguments );

	qDebug( QString( "KDCApplySubtitleJob:: " + arguments.join(" ") ).toUtf8() );
}

KDCApplySubtitleJob::KDCApplySubtitleJob( int id, const KDCMovieGroup *group, QObject *parent )
	: KDCJob( parent )
{
	p_id = id + 1;
	p_oldFrame = 0;
	setTotalSize( qRound( group->length()/1000.0*group->movie( 0 )->fps() ) );
	p_fileName = KDCFileFactory::movieName( group );
	p_movie = group->movie( 0 );

	p_process = new KProcess( this );
	p_process->setOutputChannelMode( KProcess::SeparateChannels );
	p_process->setWorkingDirectory( KDCFileFactory::discDirectory( group->disc() ) );
	p_process->setStandardInputFile( p_fileName );
	p_fileName.chop( 4 );
	p_process->setStandardOutputFile( p_fileName + "_sub.mpg" );

	connect( p_process, SIGNAL( readyReadStandardError() ), this, SLOT( on_readyReadStandardError() ) );
	connect( p_process, SIGNAL( finished( int , QProcess::ExitStatus ) ), this, SLOT( on_finished( int , QProcess::ExitStatus ) ) );

	QStringList arguments;
	arguments << "-n" << QString::number( KDCConfig::niceLevel() );
	arguments << "spumux" << "-m";
	if ( group->disc()->discType() == KDC::DVD )
		arguments << "dvd";
	else
		if ( group->disc()->discType() == KDC::CVD )
			arguments << "cvd";
	else
		arguments << "svcd";

	arguments << "-s" << QString::number( id ) << "-P";
	arguments << KDCFileFactory::movieSubtitleConfigFile( group, p_id );
	p_process->setProgram( "nice", arguments );
}

KDCApplySubtitleJob::~KDCApplySubtitleJob()
{
	if ( p_process )
	{
		p_process->disconnect();
		if ( p_process->state() == QProcess::Running )
			p_process->close();
		delete p_process;
	}
	qDebug( "~KDCApplySubtitleJob" );
}

void KDCApplySubtitleJob::start()
{
	QFile file( p_fileName + ".mpg" );
	p_const = (double)totalSize()/file.size();
	qDebug( "Size = %d, Frame = %d", file.size(), totalSize() );
	qDebug( QString( "KDCApplySubtitleJob::fileSize = " + QString::number( p_const ) ).toUtf8() );
	emit jobMainProgram( "spumux" );
	emit jobTotalSize( totalSize() );
	emit percent( 0 );
	emit infoMessage( i18n("Start applying subtitle stream ( %1 )").arg( p_id ) );
	p_process->start();
	p_timerID = startTimer( KDCConfig::updateData() );
	qDebug( "KDCApplySubtitleJob::start()" );
}

void KDCApplySubtitleJob::on_readyReadStandardError()
{
/*
	QByteArray tmp( p_process->readAllStandardError() );
	int index = tmp.indexOf( "byt", 8 );
	if ( index != -1 )
	{
		QByteArray data;
		index -= 2;
		char byte = tmp.at( index );
		while ( byte != ' ' )
		{
			data.prepend( byte );
			--index;
			byte = tmp.at( index );
		}
		qlonglong size = data.toLongLong();
		int frame = qRound( size*p_const );
		emit progress( frame );
		emit speed( frame - p_oldFrame );
		p_oldFrame = frame;
	}
	else
		if ( tmp.indexOf( "ERR: Couldn't load file" ) != -1 )
		{
			setFlag( 1, TRUE );
			setError( i18n("Couldn't load subtitle file.") );
			p_process->close();
		}
*/
}

void KDCApplySubtitleJob::on_finished( int exitCode, QProcess::ExitStatus exitStatus )
{
	killTimer( p_timerID );
	qDebug( "Codec = %d, status = %d", exitCode, exitStatus );
	if ( exitStatus == QProcess::CrashExit && !flag( 1 ) )
	{
		setError( i18n("There was an error during process. Error code is %1").arg( exitCode ) );
		KDCFileFactory::removeFile( p_fileName + "_sub.mpg" );
	}
	else
		if ( exitStatus == QProcess::CrashExit && flag( 1 ) )
			KDCFileFactory::removeFile( p_fileName + "_sub.mpg" );
	else
	{
		KDCFileFactory::removeFile( p_fileName + ".mpg" );
		if ( !KDCFileFactory::renameFile( p_fileName + "_sub.mpg", p_fileName + ".mpg" ) )
			setError( i18n("There was an error during process.") );
		else
		{
			emit infoMessage( i18n("Applied subtitle stream ( %1 ) sucesfully finished").arg( p_id ) );
			emit percent( 100 );
		}
	}

	if ( p_movie->movieGroup() )
	{
		KDCFileFactory::removeFile( KDCFileFactory::movieSubtitleFile( p_movie->movieGroup(), p_id ) );
		KDCFileFactory::removeFile( KDCFileFactory::movieSubtitleConfigFile( p_movie->movieGroup(), p_id ) );
	}
	else
	{
		KDCFileFactory::removeFile( KDCFileFactory::movieSubtitleFile( p_movie, p_id ) );
		KDCFileFactory::removeFile( KDCFileFactory::movieSubtitleConfigFile( p_movie, p_id ) );
	}

	qDebug( "KDCApplySubtitleJob::end::emit finished" );
	emit finished( this );
}

void KDCApplySubtitleJob::timerEvent( QTimerEvent *event )
{
	if ( event->timerId() == p_timerID )
	{
		QByteArray tmp( p_process->readAllStandardError() );
		int index = tmp.indexOf( "byt", 8 );
		if ( index != -1 )
		{
			QByteArray data;
			index -= 2;
			char byte = tmp.at( index );
			while ( byte != ' ' )
			{
				data.prepend( byte );
				--index;
				byte = tmp.at( index );
			}
			qlonglong size = data.toLongLong();
			int frame = qRound( size*p_const );
			emit progress( frame );
			emit speed( frame - p_oldFrame );
			p_oldFrame = frame;
		}
		else
			if ( tmp.indexOf( "ERR: Couldn't load file" ) != -1 )
			{
				setFlag( 1, TRUE );
				setError( i18n("Couldn't load subtitle file.") );
				p_process->close();
			}
	}
}
