/***************************************************************************
 *   Copyright (C) 2007 by Marcin Swiechowicz   *
 *   marcin.swiechowicz@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "kdcchapterspreviewcompositejob.h"

#include "../kdcfilefactory.h"
#include "../kdcnamespace.h"

#include "kdcgrabframejob.h"
#include "kdcbuildframejob.h"
#include "kdcconvertframejob.h"

#include <QString>
#include <QTime>

KDCChaptersPreviewCompositeJob::KDCChaptersPreviewCompositeJob( const KDCMovie *movie, QObject *parent )
	: KDCCompositeJob( KDCCompositeJob::No, parent )
{
	p_size = 0;

	KDCGrabFrameJob *gJob;
	KDCBuildFrameJob *bJob;
	KDCConvertFrameJob *cJob;

	QString workDir = KDCFileFactory::movieDir( movie );
	KDCFileFactory::removeDirectory( workDir );
	KDCFileFactory::createDirectory( workDir );
	int chapter;
	QTime time;
	QString file;
	double ar = 4.0/3.0;
	if ( movie->videoStream()->aspectRatio() == KDC::WideScreen )
		ar = 19.0/9.0;
	for ( int i=0; i<movie->chapterGroup()->count(); ++i )
	{
		chapter = movie->chapterGroup()->chapter( i );
		gJob = new KDCGrabFrameJob( chapter, movie->fileName(), workDir, this );
		connect( gJob, SIGNAL( finished( KDCJob * ) ), this, SLOT( on_subjobFinished( KDCJob * ) ) );
		connect( gJob, SIGNAL( percent( int ) ), this, SLOT( on_subjobPercent( int ) ) );
		addSubjob( gJob );

		cJob = new KDCConvertFrameJob(	workDir, QString("00000001.jpg"), QString("tmp.png"),
																		184, qRound( 184.0 / ar ), this );
		connect( cJob, SIGNAL( finished( KDCJob * ) ), this, SLOT( on_subjobFinished( KDCJob * ) ) );
		connect( cJob, SIGNAL( percent( int ) ), this, SLOT( on_subjobPercent( int ) ) );
		addSubjob( cJob );

		file = time.addSecs( chapter ).toString("hh:mm:ss") + ".png";
		bJob = new KDCBuildFrameJob( workDir, QString("tmp.png"), file, this );
		connect( bJob, SIGNAL( finished( KDCJob * ) ), this, SLOT( on_subjobFinished( KDCJob * ) ) );
		connect( bJob, SIGNAL( percent( int ) ), this, SLOT( on_subjobPercent( int ) ) );
		addSubjob( bJob );

		p_data.append( qMakePair( workDir + '/' + file, chapter ) );
	}
}

KDCChaptersPreviewCompositeJob::KDCChaptersPreviewCompositeJob( const KDCMovieGroup *group, QObject *parent )
	: KDCCompositeJob( KDCCompositeJob::No, parent )
{
	p_size = 0;

	KDCGrabFrameJob *gJob;
	KDCBuildFrameJob *bJob;
	KDCConvertFrameJob *cJob;

	QString workDir = KDCFileFactory::movieDir( group );
	KDCFileFactory::removeDirectory( workDir );
	KDCFileFactory::createDirectory( workDir );
	int chapter;
	QTime time;
	QString file;
	double ar = 4.0/3.0;
	if ( group->movie( 0 )->videoStream()->aspectRatio() == KDC::WideScreen )
		ar = 19.0/9.0;
	int length = qRound( group->movie( 0 )->videoStream()->metaData()->length()/1000.0 );
	int num = 0;
	int ss;
	int sum = 0;
	for ( int i=0; i<group->chapterGroup()->count(); ++i )
	{
		chapter = group->chapterGroup()->chapter( i );
		ss = chapter;
		while ( ss > length )
		{
			++num;
			sum = length;
			length += qRound( group->movie( num )->videoStream()->metaData()->length()/1000.0 );
		}
		ss -= sum; 
		gJob = new KDCGrabFrameJob( ss, group->movie( num )->fileName(), workDir, this );
		connect( gJob, SIGNAL( finished( KDCJob * ) ), this, SLOT( on_subjobFinished( KDCJob * ) ) );
		connect( gJob, SIGNAL( percent( int ) ), this, SLOT( on_subjobPercent( int ) ) );
		addSubjob( gJob );

		cJob = new KDCConvertFrameJob(	workDir, QString("00000001.jpg"), QString("tmp.png"),
																		184, qRound( 184.0 / ar ), this );
		connect( cJob, SIGNAL( finished( KDCJob * ) ), this, SLOT( on_subjobFinished( KDCJob * ) ) );
		connect( cJob, SIGNAL( percent( int ) ), this, SLOT( on_subjobPercent( int ) ) );
		addSubjob( cJob );

		file = time.addSecs( chapter ).toString("hh:mm:ss") + ".png";
		bJob = new KDCBuildFrameJob( workDir, QString("tmp.png"), file, this );
		connect( bJob, SIGNAL( finished( KDCJob * ) ), this, SLOT( on_subjobFinished( KDCJob * ) ) );
		connect( bJob, SIGNAL( percent( int ) ), this, SLOT( on_subjobPercent( int ) ) );
		addSubjob( bJob );

		p_data.append( qMakePair( workDir + '/' + file, chapter ) );
	}
}

KDCChaptersPreviewCompositeJob::~KDCChaptersPreviewCompositeJob()
{

}

void KDCChaptersPreviewCompositeJob::start()
{
	firstSubjob()->start();
}

void KDCChaptersPreviewCompositeJob::on_subjobFinished( KDCJob *job )
{
	if ( job->hasErrors() )
	{
		setError( job->error() );
		removeSubjob( job );
		emit finished( this );
		deleteLater();
	}
	else
	{
		removeSubjob( job );
		if ( hasSubjobs() )
			firstSubjob()->start();
		else
		{
			emit progress( totalSize() );
			emit finished( this );
			deleteLater();
		}
	}
}

void KDCChaptersPreviewCompositeJob::on_subjobPercent( int value )
{
	int newValue = p_size + value;
	if ( p_size != newValue )
	{
		p_size = newValue;
		emit percent( p_size );

		if ( ( p_size != 0 ) && ( p_size % 300 == 0 ) )
		{
			QPair<QString,int> data = p_data.takeFirst();
			emit previewAvalible( data.first, data.second );
		}
	}
}

void KDCChaptersPreviewCompositeJob::stop()
{
	while ( hasSubjobs() )
		removeSubjob( lastSubjob() );
}
