/***************************************************************************
 *   Copyright (C) 2007 by Marcin Swiechowicz   *
 *   marcin.swiechowicz@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "kdcdecorateframejob.h"

#include "../kdcfilefactory.h"

#include <QStringList>

#include <KLocale>

KDCDecorateFrameJob::KDCDecorateFrameJob(	const QString& workDir, const QString& inputFile,
																					const QString& outputFile, QObject *parent )
	: KDCJob( parent )
{
	p_workDir = workDir;
	p_inputFile = inputFile;
	p_outputFile = outputFile;

	p_process = new KProcess( this );
	connect( p_process, SIGNAL( finished( int , QProcess::ExitStatus ) ), this, SLOT( on_finished( int , QProcess::ExitStatus ) ) );
	p_process->setWorkingDirectory( workDir );
	p_process->setOutputChannelMode( KProcess::MergedChannels );
	connect( p_process, SIGNAL( readyReadStandardOutput() ), this, SLOT( on_readyReadStandardOutput() ) );

	QStringList arg;
	arg << inputFile << "-bordercolor" << "snow" << "-background" << "none" << "-polaroid" << "0" << outputFile;
	p_process->setProgram( "convert", arg );
	setTotalSize( 100 );
}

KDCDecorateFrameJob::~KDCDecorateFrameJob()
{
	if ( p_process )
	{
		p_process->disconnect();
		if ( p_process->state() == QProcess::Running )
			p_process->close();
		delete p_process;
	}
}

void KDCDecorateFrameJob::on_readyReadStandardOutput()
{
	QString error( p_process->readAllStandardOutput() );
	setFlag( 1, TRUE );
	setError( error );
	p_process->close();
}

void KDCDecorateFrameJob::start()
{
	emit percent( 0 );
	emit infoMessage( i18n("Decorating frame...") );
	p_process->start();
}

void KDCDecorateFrameJob::on_finished( int code , QProcess::ExitStatus status )
{
	qDebug( "KDCDecorateFrameJob::finished()::Codec = %d, status = %d", code, status );
	if ( status == QProcess::CrashExit && !flag( 1 ) )
		setError( i18n("There was an error during process. Error code is %1").arg( code ) );
	else
		emit percent( 100 );

	KDCFileFactory::removeFile( p_workDir + '/' + p_inputFile );
	emit finished( this );
}
