/***************************************************************************
 *   Copyright (C) 2007 by Marcin Swiechowicz   *
 *   marcin.swiechowicz@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "kdcmovieprocesscompositejob.h"

#include "../kdcsubtitlestreamgroup.h"
#include "../kdcdisc.h"
#include "../kdcfilefactory.h"
#include "kdcsubtitleconvertjob.h"
#include "kdcmovieconvertjob.h"
#include "kdcapplysubtitlejob.h"
#include "kdcsubtitlecreatexmlfilejob.h"

#include <KLocale>

KDCMovieProcessCompositeJob::KDCMovieProcessCompositeJob( const KDCMovie* movie , QObject *parent )
	: KDCCompositeJob( KDCCompositeJob::Yes, parent )
{
	//first we must create movie
	KDCMovieConvertJob *eJob = new KDCMovieConvertJob( movie );
	connect( eJob, SIGNAL( finished( KDCJob * ) ), this, SLOT( on_jobFinished( KDCJob * ) ) );
	addSubjob( eJob );

	p_file = KDCFileFactory::fileName( movie->fileName() );
	//next we convert subtitles, if there exists
	if ( movie->subtitleStreamGroup()->count() > 0 )
	{
		KDCSubtitleStreamGroup *group = movie->subtitleStreamGroup();
		KDCApplySubtitleJob *sJob;
		KDCSubtitleConvertJob *cJob;
		KDCSubtitleCreateXMLFileJob *xJob;

		for ( int i=0; i<group->count(); ++i )
		{
			// first we must convert subtitle to correct fps
			cJob = new KDCSubtitleConvertJob( i, movie, 0, this );
			connect( cJob, SIGNAL( finished( KDCJob * ) ), this, SLOT( on_jobFinished( KDCJob * ) ) );
			addSubjob( cJob );

			// next we must create xml file for spumux
			xJob = new KDCSubtitleCreateXMLFileJob( i, movie, this );
			connect( xJob, SIGNAL( finished( KDCJob * ) ), this, SLOT( on_jobFinished( KDCJob * ) ) );
			addSubjob( xJob );

			sJob = new KDCApplySubtitleJob( i, movie, this );
			connect( sJob, SIGNAL( finished( KDCJob * ) ), this, SLOT( on_jobFinished( KDCJob * ) ) );
			addSubjob( sJob );
		}
	}
}

KDCMovieProcessCompositeJob::~KDCMovieProcessCompositeJob()
{
	qDebug( "~KDCMovieProcessCompositeJob" );
}

void KDCMovieProcessCompositeJob::start()
{
	emit infoMessage( i18n("Start processing file ") + p_file );
	firstSubjob()->start();
}

void KDCMovieProcessCompositeJob::on_jobFinished( KDCJob *job )
{
	qDebug( "KDCMovieProcessCompositeJob::jobFinished()" );
	if ( job->hasErrors() )
	{
		qDebug( "Job has error" );
		emit errorMessage( job->error() );
		removeSubjob( job );
		setError( "0" );
		emit finished( this );
	}
	else
	{
		removeSubjob( job );
		if ( hasSubjobs() )
			firstSubjob()->start();
		else
		{
			emit infoMessage( i18n("Processing file ") + p_file + i18n(" sucesfully finished") );
			emit finished( this );
		}
	}
}
