/***************************************************************************
 *   Copyright (C) 2007 by Marcin Swiechowicz   *
 *   marcin.swiechowicz@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "kdcdiscitemdelegate.h"

#include "kdcnamespace.h"

#include <QPainter>
#include <QTextDocument>
#include <QAbstractTextDocumentLayout>
#include <QStringList>
#include <QFont>
#include <QPalette>

#include <KLocale>

KDCDiscItemDelegate::KDCDiscItemDelegate( QObject *parent )
	: QItemDelegate( parent )
{
	p_drawItem = NULL;
}

KDCDiscItemDelegate::~KDCDiscItemDelegate()
{

}

void KDCDiscItemDelegate::setDrawItem( const QTreeWidgetItem *item )
{
	p_drawItem = item;
}

void KDCDiscItemDelegate::drawDisplay( QPainter * painter, const QStyleOptionViewItem & option, const QRect & rect, const QString & text ) const
{
	painter->save();
	QTextDocument doc;
	QString str;
	QColor c = option.palette.color( QPalette::HighlightedText );

	if ( text.at( 0 ) == 'V' )
	{
		if ( p_drawItem )
		{
			str.append( "<font color=""" + c.name() + """>" + text + "</font>" );
			doc.setHtml( str );
		}
		else
			doc.setPlainText( text );
		painter->translate( rect.x(), rect.y() );
	}
	else
		if ( text.at( 0 ) == 'm' )
		{
			QStringList val = text.split( '\t' );
			if ( p_drawItem )
			{
				str.append( "<table><td><font color=""" + c.name() + """>" );
				str.append( val.at( 1 ) );
				str.append( "</font></td>" );
			}
			else
			{
				str.append( "<table><td>" );
				str.append( val.at( 1 ) );
				str.append( "</td>" );
			}
			str.append( "<tr><td><font size=""-1"" color=""#777777"">");
			str.append( val.at( 2 ) + "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;" + val.at( 3 ) ); 
			str.append( "</font></td></table>" );
			doc.setHtml( str );
			painter->translate( rect.x(), rect.y() - 3 );
		}
	else
		if ( text.at( 0 ) == 'g' )
		{
			QStringList val = text.split( '\t' );
			if ( p_drawItem )
			{
				str.append( "<table><td><font color=""" + c.name() + """>" );
				str.append( val.at( 2 ) );
				str.append( "</font></td>" );
			}
			else
			{
				str.append( "<table><td>" );
				str.append( val.at( 2 ) );
				str.append( "</td>" );
			}
			int size = val.at( 1 ).toInt();
			for ( int i=0; i<size; ++i )
			{
				str.append( "<tr><td><font size=""-1"" color=""#777777"">" );
				str.append( val.at( 3 + i ).mid( 1 ) );
				str.append( "</font></td>" );
			}
			str.append( "</table>" );
			doc.setHtml( str );
			painter->translate( rect.x(), rect.y() - 3 );
		}
	else
	{
		QFont f;
		f.setPointSizeF( 7.5 );
		doc.setDefaultFont( f );
		QStringList val = text.split( '\t' );

		uint count = 0;
		uint i;
		QList<QString>::const_iterator it;
		for ( it = val.constBegin(); it != val.constEnd(); ++it )
			if ( (*it) == "a" )
				++count;

		str.append( "<table><td rowspan=""7""><center><img src=" );
		str.append( val.at( 1 ) );
		str.append( " width=""184"" height=""140"" ></img></center></td>" );
		str.append( "<th colspan=""2""><font size=""+1"" color=""#366CA2""><b>Video</b></font></th><hr>" ); //video
		for ( i=0; i<count; ++i )
		{
			str.append( "<td>&nbsp;&nbsp;&nbsp;&nbsp;</td>" );
			str.append( "<th colspan=""2""><font size=""+1"" color=""#366CA2""><b>Audio</b></th></font><hr>" ); //audio
		}
		str.append( "<tr>" );
		str.append( "<td><font color=""#777777"">" + i18n("System: ") + "</font></td>" );
		if ( p_drawItem )
			str.append( "<td><font color=""" + c.name() + """>" + val.at( 4 ) + "</font></td>" ); //video
		else
			str.append( "<td>" + val.at( 4 ) + "</td>" ); //video
		for ( i=0; i<count; ++i )
		{
			str.append( "<td>&nbsp;&nbsp;&nbsp;&nbsp;</td>" );
			str.append( "<td><font color=""#777777"">" + i18n("Format: ") + "</font></td>" );
			if ( p_drawItem )
				str.append( "<td><font color=""" + c.name() + """>" + val.at( 11+i*5 ) + "</font></td>" ); //audio
			else
				str.append( "<td>" + val.at( 11+i*5 ) + "</td>" ); //audio
		}
		str.append( "<tr>" );
		str.append( "<td><font color=""#777777"">" + i18n("Fps: ") + "</font></td>" );
		if ( p_drawItem )
			str.append( "<td><font color=""" + c.name() + """>" + val.at( 5 ) + "</font></td>" ); //video
		else
			str.append( "<td>" + val.at( 5 ) + "</td>" ); //video
		for ( i=0; i<count; ++i )
		{
			str.append( "<td>&nbsp;&nbsp;&nbsp;&nbsp;</td>" );
			str.append( "<td><font color=""#777777"">" + i18n("Channels: ") + "</font></td>" );
			if ( p_drawItem )
				str.append( "<td><font color=""" + c.name() + """>" + val.at( 12+i*5 ) + "</font></td>" ); //audio
			else
				str.append( "<td>" + val.at( 12+i*5 ) + "</td>" ); //audio
		}
		str.append( "<tr>" );
		str.append( "<td><font color=""#777777"">" + i18n("Ratio: ") + "</font></td>" );
		if ( p_drawItem )
			str.append( "<td><font color=""" + c.name() + """>" + val.at( 8 ) + "</font></td>" ); //video
		else
			str.append( "<td>" + val.at( 8 ) + "</td>" ); //video
		for ( i=0; i<count; ++i )
		{
			str.append( "<td>&nbsp;&nbsp;&nbsp;&nbsp;</td>" );
			str.append( "<td><font color=""#777777"">" + i18n("Bitrate: ") + "</font></td>" );
			if ( p_drawItem )
				str.append( "<td><font color=""" + c.name() + """>" + val.at( 13+i*5 ) + " kbps</font></td>" ); //audio
			else
				str.append( "<td>" + val.at( 13+i*5 ) + " kbps</td>" ); //audio
		}
		str.append( "<tr>" );
		str.append( "<td><font color=""#777777"">" + i18n("Resolution: ") + "</font></td>" );
		if ( p_drawItem )
			str.append( "<td><font color=""" + c.name() + """>" + val.at( 7 ) + "</font></td><tr>" ); //video
		else
			str.append( "<td>" + val.at( 7 ) + "</td><tr>" ); //video

		str.append( "<td><font color=""#777777"">" + i18n("Bitrate: ") + "</font></td>" );
		if ( p_drawItem )
			str.append(" <td><font color=""" + c.name() + """>" + val.at( 6 ) + " kbps</font></td></table>" ); //video
		else
			str.append(" <td>" + val.at( 6 ) + " kbps</td></table>" ); //video
		doc.setHtml( str );
		painter->translate( rect.x(), rect.y() + 2 );
	}

	QAbstractTextDocumentLayout::PaintContext context;
	doc.setPageSize( rect.size()*=8 );
	doc.documentLayout()->draw( painter, context );
	painter->restore();
}
