/***************************************************************************
 *   Copyright (C) 2007 by Marcin Swiechowicz   *
 *   marcin.swiechowicz@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "kdcmoviegroup.h"

#include "kdcmovie.h"
#include "kdcdisc.h"
#include "kdcfilefactory.h"

#include <QStringList>
#include <QVector>
#include <QVariant>

#include <KIcon>
#include <KLocale>
#include <kio/job.h>

KDCMovieGroup::KDCMovieGroup()
	: QObject( 0 ),QTreeWidgetItem( KDC::MovieGroup )
{
	setIcon( 0, KIcon("applications-multimedia") );
	setToolTip( 0, i18n("Movie Clip") );
	QTreeWidgetItem *item = new QTreeWidgetItem( KDC::MovieInfo );
	addChild( item );
	setFlags( Qt::ItemIsEnabled | Qt::ItemIsDragEnabled | Qt::ItemIsDropEnabled );
	item->setFlags( Qt::ItemIsEnabled | Qt::ItemIsDragEnabled | Qt::ItemIsDropEnabled );
	item->setToolTip( 0, i18n("Movie Clip") );
	p_chapterGroup = NULL;
}

KDCMovieGroup::~KDCMovieGroup()
{
	QList<KDCMovie*>::const_iterator it;
	for ( it = p_movies.constBegin(); it != p_movies.constEnd(); ++it )
		delete ( *it );
}

KDCDisc* KDCMovieGroup::disc() const
{
	return p_disc;
}

KDCChapterGroup* KDCMovieGroup::chapterGroup() const
{
	return p_chapterGroup;
}

int KDCMovieGroup::id() const
{
	return p_id;
}

bool KDCMovieGroup::hasErrors()
{
	qDebug( "KDCMovieGroup::hasErrors()::begin" );
	QList<KDCMovie*>::const_iterator it = p_movies.constBegin();
	KDCMovie *movie = ( *it );
	++it;
	bool res = FALSE;
	while ( it != p_movies.constEnd() )
	{
		if ( *( *it ) != *movie )
		{
			res = TRUE;
			break;
		}
		++it;
	}
	qDebug( "KDCMovieGroup::hasErrors()::end::res = %d", QVariant( res ).toInt() );
	return res;
}

KDCMovie* KDCMovieGroup::takeMovie( int index )
{ 
	KDCMovie *tmp = p_movies.takeAt( index );
	tmp->p_group = NULL;
	refresh();
	if ( index == 0 )
	{
		disconnect( tmp, SIGNAL( settingsChanged() ), this, SLOT( refresh() ) );
		connect( p_movies.first(), SIGNAL( settingsChanged() ), this, SLOT( refresh() ) );
	}
	return tmp;
}

quint64 KDCMovieGroup::size() const
{
	return p_size;
}

void KDCMovieGroup::refresh()
{
	qDebug( "KDCMovieGroup::refresh()::begin:count = %d", p_movies.count() );
	p_size = 0;
	QList<KDCMovie*>::iterator it;
	int index = 0;
	for ( it = p_movies.begin(); it != p_movies.end(); ++it )
	{
		if ( index > 0 )
			( *it )->copySettings( p_movies.first() );
		p_size += ( *it )->size();
		++index;
	}
	KDCMovie *movie = p_movies.first();
	QString info;
	info.append( "i\t" + KDCFileFactory::movieFrame( movie ) + '\t' );
	info.append( movie->videoStream()->info() );

	qDebug( "KDCMovieGroup::refresh()::audio" );
	if ( movie->audioStreamGroup() )
	{
		KDCAudioStream *audio;
		for( int i=0; i<movie->audioStreamGroup()->count(); ++i )
		{
			audio = movie->audioStreamGroup()->stream( i );
			info.append( '\t' );
			info.append( audio->info() );
		}
	}
	child( 0 )->setText( 0, info + "\n\n\n\n\n\n\n\n\n" );
	setOutputName( p_outputName );
	qDebug( "KDCMovieGroup::refresh()::end" );
}

QString KDCMovieGroup::outputName() const
{
	return p_outputName;
}

void KDCMovieGroup::addMovie( KDCMovie *movie )
{
	movie->p_group = this;
	movie->p_disc = p_disc;
	p_movies.append( movie );
	if ( p_movies.count() > 1 )
	{
		if ( *p_movies.first()->videoStream() != *p_movies.last()->videoStream() )
		{
			p_movies.first()->setUpdatesMode( KDC::Disabled );
			p_movies.first()->videoStream()->setMode( KDC::Resample );
			p_movies.first()->setUpdatesMode( KDC::Enabled );
		}
		if ( p_movies.first()->audioStreamGroup() )
		{
			KDCAudioStream *stream1;
			KDCAudioStream *stream2;
			for ( int i=0; i<p_movies.first()->audioStreamGroup()->count(); ++i )
			{
				stream1 = p_movies.first()->audioStreamGroup()->stream( i );
				stream2 = p_movies.last()->audioStreamGroup()->stream( i );
				if ( *stream1 != *stream2 )
				{
					stream1->setUpdatesMode( KDC::Disabled );
					stream1->setMode( KDC::Resample );
					stream1->setChannelCount( qMin( stream1->channelCount(), stream2->channelCount() ) );
					stream1->setUpdatesMode( KDC::Enabled );
				}
			}
		}
	}
	refresh();
	if ( p_movies.first() == movie )
		connect( p_movies.first(), SIGNAL( settingsChanged() ), this, SLOT( refresh() ) );
}

void KDCMovieGroup::removeMovie( KDCMovie *movie )
{
	if ( p_movies.first() == movie )
		disconnect( movie, SIGNAL( settingsChanged() ), this, SLOT( refresh() ) );
	movie->p_group = NULL;
	p_movies.removeAll( movie );
	refresh();
}

void KDCMovieGroup::setOutputName( const QString& name )
{
	p_outputName = name;
	QString str = "g\t" + QString::number( p_movies.size() ) + '\t' + p_outputName;
	QList<KDCMovie*>::const_iterator it;
	for ( it = p_movies.constBegin(); it != p_movies.constEnd(); ++it )
	{
		str.append( "\t\n" );
		str.append( (*it)->fileName() );
		str.append( "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;" );
		str.append( KIO::convertSizeFromKiB( (*it)->size() ) );
	}
	//str.append( '\t' + KIO::convertSizeFromKiB( p_size ) );
	setText( 0, str );
}

KDCMovie* KDCMovieGroup::movie( int index ) const
{
	return p_movies.at( index );
}

int KDCMovieGroup::count() const
{
	return p_movies.count();
}

QList<QTreeWidgetItem*> KDCMovieGroup::takeMovies()
{
	QList<QTreeWidgetItem*> list;
	KDCMovie *tmp;
	while( !p_movies.isEmpty() )
	{
		tmp = p_movies.takeFirst();
		tmp->p_group = NULL;
		list.append( tmp );
		disconnect( tmp, SIGNAL( settingsChanged() ), this, SLOT( refresh() ) );
	}
	return list;
}

int KDCMovieGroup::length() const
{
	int size = 0;
	QList<KDCMovie*>::const_iterator it;
	for ( it = p_movies.constBegin(); it != p_movies.constEnd(); ++it )
		size += ( *it )->videoStream()->metaData()->length();
	return size;
}

QStringList KDCMovieGroup::codeMovies()
{
	QStringList list;

	int i = 0;
	QList<KDCMovie*>::const_iterator it;
	for ( it = p_movies.constBegin(); it != p_movies.constEnd(); ++it )
		list.append( QString( (*it)->fileName() + "\t%1" ).arg( i++ ) );

	return list;
}

void KDCMovieGroup::move( int from, int to )
{
	if ( from != to )
	{
		qDebug( "KDCMovieGroup::move( %d, %d )", from, to );
		if ( from == 0 || to == 0 )
		{
			disconnect( p_movies.first(), SIGNAL( settingsChanged() ), this, SLOT( refresh() ) );
			p_movies.move( from, to );
			connect( p_movies.first(), SIGNAL( settingsChanged() ), this, SLOT( refresh() ) );
		}
		else
			p_movies.move( from, to );
		refresh();
	}
}
