/***************************************************************************
 *   Copyright (C) 2007 by Marcin Swiechowicz   *
 *   marcin.swiechowicz@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "kdcinfowidget.h"

#include <KLocale>
#include <KIcon>

KDCInfoWidget::KDCInfoWidget( QWidget *parent ) : KTabWidget( parent )
{
	setMinimumWidth( 293 );
	setSizePolicy( QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding ) );
	clean();
	setTabPosition( QTabWidget::South );
}

KDCInfoWidget::~KDCInfoWidget()
{

}

void KDCInfoWidget::setCurrentMovie( KDCMovie *movie )
{
	if ( movie != p_movie )
	{
		qDebug( "KDCInfoWidget::setCurrentMovie()::begin" );
		clean();
		p_movie = movie;
		setTabBarHidden( TRUE );
		p_pages.append( new KDCMovieInfoWidget( movie ) );
		int index = addTab( p_pages.last(), KIcon("document-properties") , i18n("Movie") );
		setTabToolTip( index, movie->fileName() );
		qDebug( "KDCInfoWidget::setCurrentMovie()::end" );
	}
}

void KDCInfoWidget::setCurrentMovie( KDCMovieGroup *group )
{
	if ( group != p_group )
	{
		qDebug( "KDCInfoWidget::setCurrentMovieGroup()::begin" );
		clean();
		p_group = group;
		int index;
		KDCMovie *tmp;
		setTabBarHidden( FALSE );
		for ( int i=0; i<group->count(); ++i )
		{
			tmp = group->movie( i );
			p_pages.append( new KDCMovieInfoWidget( tmp ) );
			index = addTab( p_pages.last(), KIcon("document-properties"), i18n("Movie %1").arg( i + 1 ) );
			setTabToolTip( index, tmp->fileName() );
		}
		qDebug( "KDCInfoWidget::setCurrentMovieGroup()::end" );
	}
}

void KDCInfoWidget::clean()
{
	qDebug( "KDCInfoWidget::clean()::begin:: count = %d", p_pages.count() );
	p_movie = NULL;
	p_group = NULL;
	while( !p_pages.isEmpty() )
	{
		removeTab( 0 );
		delete p_pages.takeFirst();
	}
	qDebug( "KDCInfoWidget::clean()::end" );
}
