/***************************************************************************
 *   Copyright (C) 2007 by Marcin Swiechowicz   *
 *   marcin.swiechowicz@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "kdclabel.h"

#include <QFont>
#include <QPalette>

KDCLabel::KDCLabel( Mode mode, QWidget *parent ) : QLabel( parent )
{
	setCurrentMode( mode );
}

KDCLabel::KDCLabel( const QString& str, Mode mode, QWidget *parent )
	: QLabel( str, parent )
{
	setCurrentMode( mode );
}

KDCLabel::KDCLabel( const QString& str, const QColor& color, Mode mode, QWidget *parent )
	: QLabel( str, parent )
{
	setCurrentMode( mode );
	setColor( color );
}

KDCLabel::~KDCLabel()
{

}

void KDCLabel::setCurrentMode( Mode mode )
{
	if ( mode == BLACKTITLE )
	{
		QFont font = this->font();
		font.setBold( TRUE );
		setFont( font );
		QPalette palette = this->palette();
		palette.setColor( QPalette::Window, QColor::fromRgb( 194, 194, 194 ) );
		setPalette( palette );
		setAutoFillBackground( TRUE );
	}
	else
		if ( mode == FRAME )
			setFrameShape( QFrame::StyledPanel );
	else
		if ( mode == TITLE )
		{
			QFont font = this->font();
			font.setBold( TRUE );
			setFont( font );
		}
	else
		if ( mode == INFO )
		{
			QFont font = this->font();
			font.setBold( FALSE );
			setFont( font );
			QPalette palette = this->palette();
			palette.setColor( QPalette::Window, QColor::fromRgb( 238, 238, 52 ) );
			setPalette( palette );
			setAutoFillBackground( TRUE );
		}
	else
		{
			QFont font = this->font();
			font.setBold( FALSE );
			setFont( font );
			setAutoFillBackground( FALSE );
		}
}

void KDCLabel::setPointSize( uint size )
{
	QFont font = this->font();
	font.setPointSize( size );
	setFont( font );
}

void KDCLabel::mousePressEvent( QMouseEvent * event )
{
	emit clicked();
}

void KDCLabel::setData( QVariant data )
{
	p_data = data;
}

QVariant KDCLabel::data() const
{
	return p_data;
}

void KDCLabel::setColor( const QColor& color )
{
	QPalette p = this->palette();
	p.setColor( QPalette::Active, QPalette::WindowText, color );
	setPalette( p );
}
