/***************************************************************************
 *   Copyright (C) 2007 by Marcin Swiechowicz   *
 *   marcin.swiechowicz@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "kdcmainwindowframe.h"

#include "../kdcnamespace.h"
#include "../lib/kdclib.h"

#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QVariant>

#include <KIcon>
#include <KLocale>

KDCMainWindowFrame::KDCMainWindowFrame( KDCDiscFactory *discFactory, QWidget *parent ) : QWidget( parent )
{
	int max = 26;

	p_hTab = new KMultiTabBar( KMultiTabBar::Right );
	p_hTab->setStyle( KMultiTabBar::KDEV3ICON );
	p_hTab->setMinimumWidth( max );
	p_hTab->setMaximumWidth( max );

	p_hTab->appendTab( KIcon("kmplayer").pixmap( max, max ), 0, i18n("Media Player") );
	p_hTab->tab( 0 )->setPosition( KMultiTabBar::Right );
	p_hTab->tab( 0 )->setStyle( KMultiTabBar::KDEV3ICON );

	p_hTab->appendTab( KIcon("help-about").pixmap( max, max ), 1, i18n("Properties") );
	p_hTab->tab( 1 )->setPosition( KMultiTabBar::Right );
	p_hTab->tab( 1 )->setStyle( KMultiTabBar::KDEV3ICON );

	p_hButtons = new QButtonGroup( this );
	p_hButtons->setExclusive( FALSE );
	p_hButtons->addButton( p_hTab->tab( 0 ), 0 );
	p_hButtons->addButton( p_hTab->tab( 1 ), 1 );
	connect( p_hButtons, SIGNAL( buttonClicked( int ) ), this, SLOT( on_infoButtonClicked( int ) ) );

	p_vTab = new KMultiTabBar( KMultiTabBar::Bottom );
	p_vTab->setStyle( KMultiTabBar::KDEV3ICON );
	p_vTab->setMinimumHeight( max );
	p_vTab->setMaximumHeight( max );

	p_vTab->appendTab( KIcon("applications-multimedia").pixmap( max, max ), 0, i18n("Audio and Video") );
	p_vTab->tab( 0 )->setPosition( KMultiTabBar::Bottom );
	p_vTab->tab( 0 )->setStyle( KMultiTabBar::KDEV3ICON );

	p_vTab->appendTab( KIcon("fill-color").pixmap( max, max ), 1, i18n("Quality") );
	p_vTab->tab( 1 )->setPosition( KMultiTabBar::Bottom );
	p_vTab->tab( 1 )->setStyle( KMultiTabBar::KDEV3ICON );

	p_vTab->appendTab( KIcon("text-plain").pixmap( max, max ), 2, i18n("Subtitles") );
	p_vTab->tab( 2 )->setPosition( KMultiTabBar::Bottom );
	p_vTab->tab( 2 )->setStyle( KMultiTabBar::KDEV3ICON );

	p_vTab->appendTab( KIcon("edit-find").pixmap( max, max ), 3, i18n("Chapters") );
	p_vTab->tab( 3 )->setPosition( KMultiTabBar::Bottom );
	p_vTab->tab( 3 )->setStyle( KMultiTabBar::KDEV3ICON );

	p_vButtons = new QButtonGroup( this );
	p_vButtons->addButton( p_vTab->tab( 0 ), 0 );
	p_vButtons->addButton( p_vTab->tab( 1 ), 1 );
	p_vButtons->addButton( p_vTab->tab( 2 ), 2 );
	p_vButtons->addButton( p_vTab->tab( 3 ), 3 );

	connect( p_vButtons, SIGNAL( buttonClicked( int ) ), this, SLOT( on_settingsButtonClicked( int ) ) );
	p_vID = -1;

	p_hSplitter = new QSplitter( Qt::Horizontal );
	p_hSplitter->addWidget( discFactory );
	p_hStack = new QStackedWidget;
	p_hStack->setVisible( FALSE );
	p_hSplitter->addWidget( p_hStack );
	p_hSplitter->setStretchFactor( 0, 10 );

	QHBoxLayout *hLayout = new QHBoxLayout;
	hLayout->addWidget( p_hSplitter );
	hLayout->addWidget( p_hTab );

	QWidget *hWidget = new QWidget;
	hWidget->setLayout( hLayout );

	p_vSplitter = new QSplitter( Qt::Vertical );
	p_vSplitter->addWidget( hWidget );
	p_vStack = new QStackedWidget;
	p_vStack->setVisible( FALSE );
	p_vSplitter->addWidget( p_vStack );
	p_vSplitter->setStretchFactor( 0, 10 );

	QVBoxLayout *mainLayout = new QVBoxLayout;
	mainLayout->addWidget( p_vSplitter );
	mainLayout->addWidget( p_vTab );

	setLayout( mainLayout );

	p_discFactory = discFactory;

	connect( p_discFactory, SIGNAL( currentObjectChanged( QTreeWidgetItem * ) ), this, SLOT( on_currentObjectChanged( QTreeWidgetItem * ) ) );
	connect ( p_discFactory, SIGNAL( currentDiscChanged( KDCDisc * ) ), this, SLOT( on_discChanged( KDCDisc * ) ) );

	p_hTab->setEnabled( FALSE );
	p_vTab->setEnabled( FALSE );

	p_infoWidget = new KDCInfoWidget;
	p_infoWidget->setVisible( FALSE );
	p_mediaPlayer = new KDCMediaPlayer;
	p_mediaPlayer->setVisible( FALSE );
	p_avEditor = new KDCAudioVideoEditor;
	p_fileEditor = new KDCFileEditor;
	p_qualityEditor = new KDCQualityEditor;
	p_chapterEditor = new KDCChapterEditor( p_mediaPlayer->playerWidget() );

	QWidget *widget = new QWidget;
	QVBoxLayout *temp = new QVBoxLayout( widget );
	temp->addSpacing( 28 );
	temp->addWidget( p_mediaPlayer );
	temp->addWidget( p_infoWidget );
	temp->setStretchFactor( p_mediaPlayer, 0 );
	temp->setStretchFactor( p_infoWidget, 1 );

	p_hStack->addWidget( widget );
	p_map.insert( 0, p_mediaPlayer );
	p_map.insert( 1, p_infoWidget );

	p_vStack->addWidget( p_avEditor );
	p_vStack->addWidget( p_qualityEditor );
	p_vStack->addWidget( p_fileEditor );
	p_vStack->addWidget( p_chapterEditor );
}

KDCMainWindowFrame::~KDCMainWindowFrame()
{
	delete p_vButtons;
}

KDCDiscFactory* KDCMainWindowFrame::discFactory() const
{
	return p_discFactory;
}

void KDCMainWindowFrame::on_infoButtonClicked( int id )
{
	qDebug( "KDCMainWindowFrame::infoButtonClicked( %d )", id );
	if ( p_hTab->tab( id )->isChecked() )
	{
		p_hStack->setVisible( TRUE );
		p_map.value( id )->setVisible( TRUE );
		QTreeWidgetItem *movie = p_discFactory->currentDisc()->currentMovie();
		if ( movie )
		{
			if ( movie->type() == KDC::Movie )
			{
				if ( id == 0 )
					p_mediaPlayer->setCurrentMovie( static_cast<KDCMovie*>( movie ) );
				else
					p_infoWidget->setCurrentMovie( static_cast<KDCMovie*>( movie ) );
			}
			else
			{
				if ( id == 0 )
					p_mediaPlayer->setCurrentMovie( static_cast<KDCMovieGroup*>( movie ) );
				else
					p_infoWidget->setCurrentMovie( static_cast<KDCMovieGroup*>( movie ) );
			}
		}
		else
		{
			if ( id == 0 )
				p_mediaPlayer->clean();
			else
				p_infoWidget->clean();
		}
	}
	else
	{
		p_map.value( id )->setVisible( FALSE );
	}
	if ( !p_mediaPlayer->isVisible() && !p_infoWidget->isVisible() )
		p_hStack->setVisible( FALSE );
}

void KDCMainWindowFrame::on_currentObjectChanged( QTreeWidgetItem *item )
{
	qDebug( "KDCMainWindowFrame::currentObjectChanged( %d )", KDCLib::isNull( item ) );
	qDebug( "KDCMainWindowFrame::currentObjectChanged()::info visible = %d", QVariant( p_hStack->isVisible() ).toInt() );
	qDebug( "KDCMainWindowFrame::currentObjectChanged()::settings visible = %d", QVariant( p_vStack->isVisible() ).toInt() );
	if ( item )
	{
		qDebug( "KDCMainWindowFrame::currentObjectChanged()::item type = %d", item->type() );
		if ( item->type() == KDC::VTS )
		{
			cleanVStack();
			cleanHStack();
		}
		else
		{
			if ( p_vStack->isVisible() || p_hStack->isVisible() )
			{
				KDCMovie *movie = NULL;
				KDCMovieGroup *group = NULL;
				if ( item->type() == KDC::Movie )
					movie = static_cast<KDCMovie*>( item );
				else
					group = static_cast<KDCMovieGroup*>( item );
				if ( p_vStack->isVisible() )
				{
					if ( movie )
					{
						p_avEditor->setCurrentMovie( movie );
						p_fileEditor->setCurrentMovie( movie );
						p_qualityEditor->setCurrentMovie( movie );
						if ( movie->disc()->discType() == KDC::DVD )
							p_chapterEditor->setCurrentMovie( movie );
					}
					else
					{
						p_avEditor->setCurrentMovie( group );
						p_fileEditor->setCurrentMovie( group );
						p_qualityEditor->setCurrentMovie( group );
						if ( group->disc()->discType() == KDC::DVD )
							p_chapterEditor->setCurrentMovie( group );
					}
				}
				if ( p_hStack->isVisible() )
				{
					if ( movie )
					{
						p_infoWidget->setCurrentMovie( movie );
						p_mediaPlayer->setCurrentMovie( movie );
					}
					else
					{
						p_mediaPlayer->setCurrentMovie( group );
						p_infoWidget->setCurrentMovie( group );
					}
				}
			}
		}
	}
	else
	{
		cleanVStack();
		cleanHStack();
	}
}

void KDCMainWindowFrame::cleanVStack()
{
	p_avEditor->clean();
	p_fileEditor->clean();
	p_qualityEditor->clean();
	p_chapterEditor->clean();
}

void KDCMainWindowFrame::cleanHStack()
{
	p_infoWidget->clean();
	p_mediaPlayer->clean();
}

void KDCMainWindowFrame::on_settingsButtonClicked( int id )
{
	qDebug( "KDCMainWindowFrame::settingsButtonClicked( %d )", id );
	if ( p_vID == id && p_vTab->tab( id )->isChecked() )
	{
		p_vButtons->setExclusive( FALSE );
		p_vTab->tab( id )->setChecked( FALSE );
		p_vButtons->setExclusive( TRUE );
		p_vID = -1;
		p_vStack->setVisible( FALSE );
		//cleanVStack();
		qDebug( "KDCMainWindowFrame::settingsButtonClicked():: visible = FALSE" );
	}
	else
	{
		p_vID = id;
		if ( !p_vStack->isVisible() )
		{
			p_vStack->setVisible( TRUE );
			qDebug( "KDCMainWindowFrame::settingsButtonClicked():: visible = TRUE" );
			on_currentObjectChanged( p_discFactory->currentDisc()->currentItem() );
		}
		p_vStack->setCurrentIndex( id );
	}
}

void KDCMainWindowFrame::on_discChanged( KDCDisc *disc )
{
	qDebug( "KDCMainWindowFrame::discChanged( %d ), p_vID = %d", KDCLib::isNull( disc ), p_vID );
	if ( !disc )
	{
		p_hStack->setVisible( FALSE );
		p_hTab->tab( 0 )->setChecked( FALSE );
		p_hTab->tab( 1 )->setChecked( FALSE );
		p_hTab->setEnabled( FALSE );
		if ( p_vID != -1 )
			on_settingsButtonClicked( p_vID );
		p_vTab->setEnabled( FALSE );
	}
	else
	{
		p_hTab->setEnabled( TRUE );
		p_vTab->setEnabled( TRUE );
		p_chapterEditor->clean();
		if ( disc->discType() != KDC::DVD )
			p_chapterEditor->setEnabled( FALSE );
		else
			p_chapterEditor->setEnabled( TRUE );
	}
}
