/***************************************************************************
 *   Copyright (C) 2007 by Marcin Swiechowicz   *
 *   marcin.swiechowicz@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "kdcoutputdirstatusbar.h"

#include "../kdcfilefactory.h"

#include <QHBoxLayout>

#include <KLocale>
#include <KIcon>
#include <KDiskFreeSpace>
#include <kio/job.h>

KDCOutputDirStatusBar::KDCOutputDirStatusBar( QWidget *parent )
	: QWidget( parent )
{
	p_projectSize = 0;
	p_size = 20;
	p_caption = new KDCLabel( i18n( "Output:" ) + " 0.0 B / 0.0 B" );
	p_icon = new KDCLabel;
	p_icon->setPixmap( KIcon("folder-green").pixmap( p_size, p_size ) );
	p_mountPoint = "/";
	p_path = "/";

	QHBoxLayout *hLayout = new QHBoxLayout;
	hLayout->addWidget( p_icon );
	hLayout->addWidget( p_caption );
	hLayout->addStretch();
	setLayout( hLayout );

	p_timer = new QBasicTimer;
	refresh();
}

KDCOutputDirStatusBar::~KDCOutputDirStatusBar()
{
	p_timer->stop();
	delete p_timer;
	delete p_icon;
	delete p_caption;
}

void KDCOutputDirStatusBar::start()
{
	if ( !p_timer->isActive() )
		p_timer->start( 20000, this );
}

void KDCOutputDirStatusBar::stop()
{
	p_timer->stop();
}

void KDCOutputDirStatusBar::setProjectSize( quint64 size )
{
	setProjectInfo( p_path, size );
}

void KDCOutputDirStatusBar::reset()
{
	p_icon->setPixmap( KIcon("folder-green").pixmap( p_size, p_size ) );
	p_projectSize = 0;
}

void KDCOutputDirStatusBar::setOutputPath( const QString& path )
{
	setProjectInfo( path, p_projectSize );
}

void KDCOutputDirStatusBar::setProjectInfo( const QString& path, quint64 size )
{
	p_projectSize = size;
	p_path = path;
	p_mountPoint = KDCFileFactory::mountPoint( p_path );
	setToolTip( path );
	refresh();
}

void KDCOutputDirStatusBar::refresh()
{
	KDiskFreeSpace *freeDiskSpace = new KDiskFreeSpace( this );
	connect(	freeDiskSpace, SIGNAL( foundMountPoint( const QString&, quint64, quint64, quint64 ) ), this,
						SLOT( on_foundMountPoint( const QString&, quint64, quint64, quint64 ) ) );
	freeDiskSpace->readDF( p_mountPoint );
}

void KDCOutputDirStatusBar::on_foundMountPoint( const QString& mountPoint, quint64 kBSize, quint64 kBUsed, quint64 kBAvail )
{
	p_freeSize = kBAvail*1024;
	p_totalSize = kBSize*1024;
	p_mountPoint = mountPoint;

	if ( p_projectSize > p_freeSize )
		p_icon->setPixmap( KIcon("folder-red").pixmap( p_size, p_size ) );
	else
		if ( ( p_projectSize + 52428800 ) > p_freeSize )
			p_icon->setPixmap( KIcon("folder-orange").pixmap( p_size, p_size ) );
	else
		p_icon->setPixmap( KIcon("folder-green").pixmap( p_size, p_size ) );

	p_caption->setText( i18n("Output: ") + KIO::convertSize( p_freeSize ) + " / " + KIO::convertSize( p_totalSize ) );
}

void KDCOutputDirStatusBar::setIconSize( int size )
{
	p_size = size;
	refresh();
}

QString KDCOutputDirStatusBar::outputPath() const
{
	return p_path;
}

QString KDCOutputDirStatusBar::mountPoint() const
{
	return p_mountPoint;
}

quint64 KDCOutputDirStatusBar::freeSize() const
{
	return p_freeSize;
}

quint64 KDCOutputDirStatusBar::totalSize() const
{
	return p_totalSize;
}

quint64 KDCOutputDirStatusBar::projectSize() const
{
	return p_projectSize;
}

void KDCOutputDirStatusBar::timerEvent( QTimerEvent *event )
{
	if ( event->timerId() == p_timer->timerId() && p_timer->isActive() )
		refresh();
	else
		QWidget::timerEvent( event );
}

void KDCOutputDirStatusBar::setTextColor( const QColor& color )
{
	p_caption->setColor( color );
}
